#include "imagestatic.h"
#include <time.h>

ImageStatic::ImageStatic(QString _path)
    : Image(_path)
{
    load();
}

ImageStatic::ImageStatic(std::unique_ptr<DocumentInfo> _info)
    : Image(std::move(_info))
{
    load();
}

ImageStatic::~ImageStatic() {
}

//load image data from disk
void ImageStatic::load() {
    if(isLoaded()) {
        return;
    }
    std::unique_ptr<const QImage> img(new QImage(mPath, mDocInfo->extension()));
    // temporary workaround for all the colorMap bullshit
    if(img->format() == QImage::Format_Indexed8) {
        img.reset(new QImage(img->convertToFormat(QImage::Format_RGB888)));
    }
    img = ImageLib::exifRotated(std::move(img), this->mDocInfo.get()->exifOrientation());
    // set image
    image = std::move(img);
    mLoaded = true;
}

// TODO: add a way to configure compression level?
bool ImageStatic::save(QString destPath) {
    int quality = 95;
    return isEdited()?imageEdited->save(destPath, nullptr, quality):image->save(destPath, nullptr, quality);
}

bool ImageStatic::save() {
    int quality = 95;
    return isEdited()?imageEdited->save(mPath, nullptr, quality):image->save(mPath, nullptr, quality);
}

std::unique_ptr<QPixmap> ImageStatic::getPixmap() {
    std::unique_ptr<QPixmap> pix(new QPixmap());
    isEdited()?pix->convertFromImage(*imageEdited):pix->convertFromImage(*image);
    return pix;
}

std::shared_ptr<const QImage> ImageStatic::getSourceImage() {
    return image;
}

std::shared_ptr<const QImage> ImageStatic::getImage() {
    return isEdited()?imageEdited:image;
}

int ImageStatic::height() {
    return isEdited()?imageEdited->height():image->height();
}

int ImageStatic::width() {
    return isEdited()?imageEdited->width():image->width();
}

QSize ImageStatic::size() {
    return isEdited()?imageEdited->size():image->size();
}

bool ImageStatic::setEditedImage(std::unique_ptr<const QImage> imageEditedNew) {
    if(imageEditedNew && imageEditedNew->width() != 0) {
        discardEditedImage();
        imageEdited = std::move(imageEditedNew);
        mEdited = true;
        return true;
    } else {
        return false;
    }
}

bool ImageStatic::discardEditedImage() {
    if(imageEdited) {
        imageEdited.reset();
        mEdited = false;
        return true;
    }
    return false;
}
